/*
 * Copyright (c) 2018 Alibaba Group Holding Ltd. All rights reserved.
 */

'use strict';

var fs = require('fs');
var path = require('path');
var constant = require('../../utils/constant');
var Log = require('../../utils/log');

var TAG = 'Group';
var GROUP_CONFIG_DIR = constant.GroupConfigDir;
var GROUP_INFO_FILE = constant.jobIndex;

function Group() {
  this.infos = null;
}

Group.prototype.load = function (callback) {
  Log.i(TAG, `Reading dir "${GROUP_CONFIG_DIR}" for group config.`);
  fs.readdir(GROUP_CONFIG_DIR, function (err, files) {
    if (err) {
      callback(err);
      return;
    }
    if (files.length !== 1) {
      callback(new Error('More than one group exist!'));
      return;
    }
    Log.i(TAG, `Reading group config file: "${GROUP_CONFIG_DIR}".`);
    var infoFilePath = path.join(GROUP_CONFIG_DIR, files[0], GROUP_INFO_FILE);
    fs.readFile(infoFilePath, function (err, data) {
      if (err) {
        callback(err);
        return;
      }
      var infos;
      try {
        infos = JSON.parse(data);
      } catch (err) {
        callback(new Error(`Group info file "${infoFilePath}" is not in JSON format.`));
        return;
      }
      callback(null, infos);
    });
  });
};

Group.prototype.checkAndLoad = function (callback) {
  if (this.infos) {
    callback(null, this.infos);
    return;
  }
  var self = this;
  this.load(function (err, infos) {
    if (!err) {
      self.infos = infos;
    }
    callback(err, infos);
  });
};

Group.prototype.getThingsWithTags = function (tags, callback) {
  var loaded = function (err, infos) {
    if (err) {
      callback(err);
      return;
    }
    if (!infos) {
      callback(null, []);
      return;
    }
    var devices = infos.devices || [];
    var things = devices.filter(function (device) {
      return tags.every(function (tag) {
        return device.tags.some(function (dt) {
          return dt.key !== undefined && dt.key !== null && tag[dt.key] !== undefined
            && tag[dt.key] !== null && tag[dt.key] === dt.value;
        });
      });
    }).map(function (device) {
      // Only export limited values.
      return {
        productKey: device.productKey,
        deviceName: device.deviceName,
        tags: device.tags.map(function (tag) {
          return {
            [tag.key]: tag.value,
          };
        }),
      };
    });
    return callback(null, things);
  };
  this.checkAndLoad(loaded);
};

module.exports = Group;
module.exports.Group = Group;
