/*
 * Copyright (c) 2018 Alibaba Group Holding Ltd. All rights reserved.
 */

'use strict';

class TryResult {
  constructor(error, value) {
    this.error = error;
    this.value = value;
  }

  static newError(error) {
    return new TryResult(error, undefined);
  }

  static newValue(value) {
    return new TryResult(undefined, value);
  }
}

class TryJSON {
  static parse(text) {
    try {
      return TryResult.newValue(JSON.parse(text));
    } catch (err) {
      return TryResult.newError(err);
    }
  }
  static stringify(value) {
    try {
      return TryResult.newValue(JSON.stringify(value));
    } catch (err) {
      return TryResult.newError(err);
    }
  }
}

class TryBase64 {
  static decode(text) {
    try {
      return TryResult.newValue(Buffer.from(text, 'base64'));
    } catch (err) {
      return TryResult.newError(err);
    }
  }
}

module.exports.TryResult = TryResult;
module.exports.JSON = TryJSON;
module.exports.Base64 = TryBase64;