var dmpProxy = require('../../components/dbus/dbus-proxy/DMPProxy.js');
var cmpProxy = require('../../components/dbus/dbus-proxy/CMPProxy.js');
var log = require("../developer/log");
const TAG = "[Runner_dbus_proxy]";

var Proxy = function () {

};


/**
 * 获取设备列表
 * @param  {[string]} status [设备的状态，可选参数为:
 *                           online　：　在线设备
 *                           offline　：　离线设备
 *                           如果没有指定status，将返回所有的设备
 *                           ]
 * @return {[Object]}        [返回一个Object，　有如下格式：
 *                          {
                                "code": int, // 状态码，0表示成功，其它表示失败
                                "message": string, // 提示消息
                                "params": {
                                    "devNum": int, //@devList中设备总个数
                                    "devList": [deviceCloudId_1, deviceCloudId_2, deviceCloudId_n]
                                }
                            }
 ]
 */
Proxy.prototype.getDeviceList = function(status) {
    return dmpProxy.getDeviceList(status);
};

Proxy.prototype.getDeviceListAsync = function(status, callback) {
    dmpProxy.getDeviceListAsync(status, callback);
};
/**
 * 读取一个profile文件
 * @param  {[string]} id ［设备的cloudid］
 * @return {[string]}    [profile文件内容]
 */
Proxy.prototype.getDeviceProfile = function(id) {
    const time = Date.now();
    const profile = dmpProxy.getDeviceProfile(id);
    return profile;
};

Proxy.prototype.getDeviceProfileAsync = function(id, callback) {
    dmpProxy.getDeviceProfileAsync(id, callback);
};

/**
 * 读设备的一个属性值
 * @param  {[string]} id       [设备的cloudid]
 * @param  {[string]} property [属性名]
 * @return {[object]}          [若读取成功，返回属性值；如果失败，返回undefined]
 */
Proxy.prototype.getProperty = function(id, property) {
    return cmpProxy.getProperty(id, property);
};

Proxy.prototype.getPropertyAsync = function(id, property, callback) {
    cmpProxy.getPropertyAsync(id, property, callback);
};

Proxy.prototype.getBatchPropertiesAsync = function (id, properties, callback) {
    cmpProxy.getBatchPropertiesAsync(id, properties, callback);
};

/**
 * 设置设备属性
 * @param {[string]} id       [cloudid]
 * @param {[string]} property [属性名]
 * @param {[boolean]} value    [如果调用成功，返回true；否则返回false。(不代表执行成功！)]
 */
Proxy.prototype.setProperty = function(id, property, value) {
    return cmpProxy.setProperty(id, property, value);
};

Proxy.prototype.setPropertyAsync = function(id, property, value, callback) {
    cmpProxy.setPropertyAsync(id, property, value, callback);
};

Proxy.prototype.setBatchPropertiesAsync = function(id, properties, callback) {
    cmpProxy.setBatchPropertiesAsync(id, properties, callback);
};

Proxy.prototype.callService = function(id, method, args) {
    return cmpProxy.callService(id, method, args);
};

Proxy.prototype.callServiceAsync = function(id, method, args, callback) {
    cmpProxy.callServiceAsync(id, method, args, callback);
};

Proxy.prototype.sendSignalWithName = function (wellknowName, objPath, myinterface, signal, ...args) {
    dmpProxy.proxy.proxy.sendSignal(wellknowName, objPath, myinterface, signal, ...args);
};

Proxy.prototype.sendSignal = function (signal, ...args) {
    dmpProxy.proxy.sendSignal(signal, ...args);
};

Proxy.prototype.sendJobExecuteStatus = function(id, jobstatus) {
    log.LOGI(TAG, "JobManagerProxy.sendJobStatus: " + JSON.stringify(jobstatus));
    var args = undefined;
    if (typeof jobstatus === 'object') {
        args = JSON.stringify(jobstatus);
    } else if (typeof jobstatus === 'string') {
        args = jobstatus;
    } else {
        // args will be ignored.
    }
    return this.sendSignal("jobstatus_event", args);
};


module.exports = new Proxy();
