var fs = require("fs");
const path = require("path");
var register = require("../registry/registerDevice");
const constant = require("../../utils/constant");
var Log = require("./log");
var TAG = "developer_api_group";


function ThingManager() {
};

/**
 * 根据用户脚本传递的groupId生成对象的分组设备大对象,异步接口结果必须callback中获取
 * @param groupId
 * @param callback
 */
ThingManager.prototype.getThingsByGroupId = function (groupId, callback) {
    Log.LOGI(TAG, "getThingsByGroupId begin...groupId = ", groupId);
    if(!groupId || groupId < 0) {
        return callback(new Error('param group Error, groupId = ' + groupId));
    }
    var self = this;
    this.getConfigByGroupId(groupId, function (err, config) {
        Log.LOGI(TAG, "getConfigByGroupId err = ", err, " config = ", JSON.stringify(config));
        if(!err && config) {
            self.getThingsByConfig(config, function (err, root) {
               Log.LOGI(TAG, "getThingsByGroupId res , err =" + err + "  root = " + JSON.stringify(root) + "\n");
                return callback(err, root);
            });
        } else {
            return callback(err, null);
        }
    });
};

ThingManager.prototype.getConfigPath = function () {
    Log.LOGI(TAG, "getConfigPath begin...");
    ThingManager.CONFIG_DIR = getGroupRootDir();
    console.log(TAG, " developer groupDIR = ", ThingManager.CONFIG_DIR);
    return ThingManager.CONFIG_DIR;
};

ThingManager.prototype.getConfigByGroupId = function (groupId, callback) {
    Log.LOGI(TAG, "getConfigByGroupId begin groupId = " + groupId);
    if(!groupId || groupId < 0) {
        return callback(new Error('param group Error, groupId = ' + groupId));
    }
    var hasReturned = false;
    var self = this;
    var fileNames = [];
    ThingManager.CONFIG_DIR = getGroupRootDir();
    Log.LOGI(TAG, " developer groupDIR = ", ThingManager.CONFIG_DIR);
    try {
        fileNames = fs.readdirSync(ThingManager.CONFIG_DIR);
        fileNames.reverse();
    } catch (err) {
        Log.LOGI(TAG, err);
    }
    var config = null;
    var cnt = 0;
    for (var i = 0; i < fileNames.length; i++) {
        var fileName = fileNames[i];
        if(hasReturned == true) {
            break;
        }
        var filePath = path.join(ThingManager.CONFIG_DIR, fileName, constant.jobIndex);
        // Log.LOGI(TAG, "getConfigByGroupId filePath = "+ filePath);
        fs.readFile(filePath, 'utf8', function (err, data) {
            cnt++;
            if (err) {
                Log.LOGE(TAG, "readFile err = " + err);
            }
            var confObject = null;
            try {
                confObject = JSON.parse(data);
            } catch (e) {
                Log.LOGI("catch Exception groupConfig data = " + data + " err = " + JSON.stringify(err) + " filePath = " + filePath);
            }
            config = self.getSubConfigByIdAndConfig(groupId, confObject);
            if (!hasReturned && config) {
                self.config = config;
                hasReturned = true;
                Log.LOGI(TAG, "self.config = " + JSON.stringify(config));
                return callback(false, config);
            }
            if(!hasReturned && !config && cnt === fileNames.length) {
                Log.LOGE(TAG, 'after search can not find your groupid :', groupId, 'config groupConfig size =',cnt);
                hasReturned = true;
                return callback(new Error('after search can not find your groupid :', groupId, 'config groupConfig size =', cnt));
            }
        });
    }
    if( !config && cnt === fileNames.length) {
        Log.LOGE(TAG, 'can not find your groupid : ' + groupId + 'config groupConfig size = ' + cnt);
        hasReturned = true;
        return callback(new Error('can not find your groupid : ' + groupId + ' config groupConfig size = ' + cnt));
    }
};

/**
 * 从大的json对象中找出满足groupId要求的json对象
 * @param groupId
 * @param confObject
 * @return {*}
 */
ThingManager.prototype.getSubConfigByIdAndConfig = function (groupId, confObject) {
    // console.log("getSubConfigByIdAndConfig begin...groupId = ", groupId);
    if (!groupId || !confObject || groupId === "" || Object.keys(confObject).length === 0) {
        return null;
    }
    if (confObject.groupId === groupId) {
        // Log.LOGI("getSubConfigByIdAndConfig find it ", JSON.stringify(confObject));
        return confObject;
    } else {
        if (!confObject.groups || confObject.groups.length === 0) {
            return null;
        }
        for (var i = 0; i < confObject.groups.length; i++) {
            var result = this.getSubConfigByIdAndConfig(groupId, (confObject.groups)[i]);
            if (result) {
                return result;
            }
        }
        return null;
    }
};

/**
 * 把json对象实例化成分组设备对象
 * @param configRoot {[Object]} 要实例化的json对象
 * {
"groupId":11,
"name":"405",
"devices":[
{
"productKey":"sFbUm3CoWz2",
"deviceName":"0c150c187a31"
},
{
"productKey":"Vbc4DW4W0mv",
"deviceName":"ac0c150c187a31"
},
{
"productKey":"0LAIGKYmweU",
"deviceName":"0x00124b000f8c96a2"
},
{
"productKey":"poIi3nlqh41",
"deviceName":"601ce41f78938653"
}
],
"groups":[
{
"groupId":111,
"name":"light",
"devices":[
]
}
]
}
 * @param callback object:实例化的设备对象组
 * {
"devices":{
"sFbUm3CoWz2":{
"0c150c187a31":{
"id":"sFbUm3CoWz2_0c150c187a31",
"led1":"on",
"led2":"off",
"led3":"on",
"human_exist":"off",
"not_disturb":"on",
"clean_room":"off"
}
},
"Vbc4DW4W0mv":{},
"0LAIGKYmweU":{},
"poIi3nlqh41":{}
},
"groups":{
"light":{
"devices":{
},
"groups":{
}
}
}
}
 * */
ThingManager.prototype.getThingsByConfig = function (configRoot, callback) {
    console.log(TAG, "getThingsByConfig configRoot = " + JSON.stringify(configRoot));
    if (!configRoot || Object.keys(configRoot).length === 0) {
        return callback(new Error("config invalid"));
    }
    var self = this;
    var root = {};
    if (!configRoot.devices || configRoot.devices.length === 0) {
        return this.getThingsByGroupConfig(configRoot, root, callback);
    }
    var myDevices = root.devices = {};
    var i = 0;
    var errorIds = {};
    configRoot.devices.forEach(function(device) {
        //console.log("i=", i, "begin createThing device", JSON.stringify(device));
        register.createThing(device.productKey + "_" + device.deviceName, device.tags, function (err, thing) {
            ++i;
            //console.log("i=", i, i, i, i, i, i, "err =", err, 'thing=', JSON.stringify(thing));
            if (err) {
                Log.LOGE(TAG, "getThingsByConfig create thing " + device.productKey + "_" + device.deviceName + " Error ,Error = " + err);
                if(typeof err !== "string") {
                    err = err.toString();
                }
                errorIds[device.productKey + "_" + device.deviceName] = err;
            } else {
                if (!myDevices[device.productKey]) {
                    myDevices[device.productKey] = {};
                }
                myDevices[device.productKey][device.deviceName] = thing;
            }
            if (i === configRoot.devices.length) {
                console.log("root = ", JSON.stringify(root));
                if(Object.keys(errorIds).length > 0) {
                    var errMsg = 'create things Error, Error map :' + JSON.stringify(errorIds);
                    return callback(new Error(errMsg));
                }
                self.getThingsByGroupConfig(configRoot, root, callback);
            }
        });
    });
};

/**
 * 只针对分组config生成设备模型
 * @param configRoot
 * @param root
 * @param callback
 * @return {*}
 */
ThingManager.prototype.getThingsByGroupConfig = function (configRoot, root, callback) {
    var self = this;
    if (!configRoot.groups || configRoot.groups.length === 0) {
        return callback(false, root);
    }
    var j = 0;
    var myGroups;
    configRoot.groups.forEach(function(group) {
        self.getThingsByConfig(group, function (err, object) {
            j++;
            // console.log("j=", j, j, j, j, j, j, "err =", err);
            if(err) {
                Log.LOGE(TAG, "getThingsByConfig  Error, Error = " + err);
                return callback(err);
            } else {
                if(!myGroups) {
                    myGroups = root.groups = {};
                }
                myGroups[group.name] = object;
            }
            if (j === configRoot.groups.length) {
                return callback(false, root);
            }
        });
    });
};

/**
 * 获取系统登录的HOME路径
 * @return {*}
 */
function getHome() {
    return process.env.HOME;
};

function getGroupRootDir() {
    return constant.GroupConfigDir;
}

module.exports = new ThingManager();
