const Tag = require('../../utils/constant');
const log = require('../developer/log');
const util = require('util');
const proxy = require("../ws");
const valuecomparer = require('../../components/ifttt/comparer.js');
const TAG = "Runner_IFTTT_condition";

var Condition = function () {

};

/**
 * "conditions":[
 {
     "uri":"condition/device/status",
     "params":{
         "productKey":"",
         "deviceName":"",
         "propertyName":"",
         "compareType":"",
         "compareValue":""
     }
 }
 ]
 * @param conditions
 */
Condition.prototype.isConditionsMeeted = function (conditions, callback) {
    if( !conditions || conditions.length === 0) return callback(null, true);
    var meet = false;
    var hasReturn = false;
    var cnt = 0;
    conditions.forEach(function (condi) {
        if(!condi.uri || !condi.params) {
            hasReturn = true;
            return callback("params error, condition = " + JSON.stringify(condi), false);
        }
        switch (condi.uri) {
            case Tag.DeviceCondition : {
                var id = condi.params.productKey + "_" + condi.params.deviceName;
                var key = condi.params.propertyName;
                proxy.getProperty(id, [key], function(err, params) {
                    var value = params[key];
                    var satisfy = false;
                    if (!err) {
                        log.LOGI(TAG, "origin value : " + value + " compareType : " + condi.params.compareType + " compareValue : " + condi.params.compareValue);
                        satisfy = valuecomparer(condi.params.compareType, value, condi.params.compareValue) //compare, value, checker
                        log.LOGI(TAG, "valuecomparer satisfy = " + satisfy);
                    }
                    if (!hasReturn && !satisfy) {
                        hasReturn = true;
                        return callback(err, satisfy);
                    }
                    cnt++;
                    if ( !hasReturn && cnt === conditions.length) {
                        meet = true;
                        hasReturn = true;
                        return callback(null, meet);
                    }
                });
                break;
            }
            case Tag.TimeCondition : {
                var format = condi.params.format; //"HH:mm"
                // var timezoneID = condi.params.timezoneID;
                var timezoneID = (new Date()).getTimezoneOffset() / 60;
                log.LOGI(TAG, "timezoneID = " + timezoneID);
                var beginDate = condi.params.beginDate;
                var endData = condi.params.endDate;
                // var date = getLocalTime(timezoneID);
                var dateStr = new Date().Format(format);
                log.LOGI(TAG, "getLocalTime = " + dateStr);
                log.LOGI(TAG, "beginDate = " + beginDate + " endData = " + endData + " now time = " + dateStr);
                try {
                    var currDate = timeToSeconds(dateStr);
                    if(currDate >= timeToSeconds(beginDate) && currDate <= timeToSeconds(endData)) {
                        cnt++;
                    } else {
                        hasReturn = true;
                        return callback(null, false);
                    }
                } catch (err) {
                    hasReturn = true;
                    return callback(err, false);
                }
                break;
            }
            default : {
                log.LOGE(util.format("[IFTTT::Scene::isConditionsMeeted] uri match null "));
                hasReturn = true;
                return callback("isConditionsMeeted uri match null", false);
                break;
            }
        }
        if ( !hasReturn && cnt === conditions.length) {
            meet = true;
            hasReturn = true;
            return callback(null, meet);
        }
    });
};

function timeToSeconds(time) {
    var splits = time.split(':');
    if (splits.length < 2 && splits.length > 3)
        throw 'time format must `HH:MM` or `HH:MM:SS`';

    if (splits.length === 2)
        return parseInt(splits[0]) * 3600 + parseInt(splits[1]) * 60;

    return parseInt(splits[0]) * 3600 + parseInt(splits[1]) * 60 + parseInt(splits[2]);
}

/**
 * 得到标准时区的时间的函数
 * @param {int} 时区id
 * @return {Date}
 */
function getLocalTime(id) {
    //参数i为时区值数字，比如北京为东八区则输进8,西5输入-5
    if (typeof id !== 'number') return;
    var d = new Date();
    //得到1970年一月一日到现在的秒数
    var len = d.getTime();
    //本地时间与GMT时间的时间偏移差
    var offset = d.getTimezoneOffset() * 60000;
    //得到现在的格林尼治时间
    var utcTime = len + offset;
    return new Date(utcTime + 3600000 * id);//.getLocalTime()
}

// 对Date的扩展，将 Date 转化为指定格式的String
// 月(M)、日(d)、小时(h)、分(m)、秒(s)、季度(q) 可以用 1-2 个占位符，
// 年(y)可以用 1-4 个占位符，毫秒(S)只能用 1 个占位符(是 1-3 位的数字)
// 例子：
// (new Date()).Format("yyyy-MM-dd hh:mm:ss.S") ==> 2006-07-02 08:09:04.423
// (new Date()).Format("yyyy-M-d h:m:s.S")      ==> 2006-7-2 8:9:4.18
Date.prototype.Format = function(fmt)
{
    var o = {
        "M+" : this.getMonth()+1,                 //月份
        "d+" : this.getDate(),                    //日
        "h+" : this.getHours()%12 == 0 ? 12 : this.getHours()%12, //小时
        "H+" : this.getHours(), //小时
        "m+" : this.getMinutes(),                 //分
        "s+" : this.getSeconds(),                 //秒
        "q+" : Math.floor((this.getMonth()+3)/3), //季度
        "S"  : this.getMilliseconds()             //毫秒
    };
    if(/(y+)/.test(fmt))
        fmt=fmt.replace(RegExp.$1, (this.getFullYear()+"").substr(4 - RegExp.$1.length));
    for(var k in o)
        if(new RegExp("("+ k +")").test(fmt))
            fmt = fmt.replace(RegExp.$1, (RegExp.$1.length==1) ? (o[k]) : (("00"+ o[k]).substr((""+ o[k]).length)));
    return fmt;
};

module.exports = new Condition();