#!/usr/bin/env python
#-*- coding: utf-8 -*-

from ws4py.client.threadedclient import WebSocketClient
import threading
import json
import config
import utils
import base64
import logging


class WS_Client(WebSocketClient):
    def __init__(self, url):
        super().__init__(url)
        self.connect()
        t = threading.Thread(target=self.ws_loop, args=())
        t.setDaemon(True)
        t.start()
        self.t = t
        self.uuid = 0
        self.pendingCalls = {}

    def ws_loop(self):
        self.run_forever()

    def closed(self, code, reason=None):
        logging.error("Closed down, %s, %s", code, reason)

    def received_message(self, data):
        #TODO: need to add message type
        # msg: {"tag": uuid, "status": "success", "data": data}
        # msg: {functionId: "", event: {payload: {}, topic: " " }}
        msg = json.loads(data.data.decode("utf-8"))
        logging.debug("receive_msg:%s", msg)
        if "tag" in msg:
            uuid = msg["tag"]
            status = msg["status"]
            if status == 'success':
                data = msg["value"]
            else:
                data = msg["error"]
            try:
                cb = self.pendingCalls[uuid]
                if cb is not None:
                    cb(status, data)
                    self.pendingCalls.pop(uuid)
            except Exception as e:
                logging.error("receive message, no match uuid")
        # elif "functionId" in msg:
        #     functionId = msg["functionId"]
        #     newMsg = msg["event"]
        #     status, job = jobmgr.get_job_mgr().get_job(functionId)
        #     if status:
        #         handler = job.func
        #         result = job.status
        #         context = job.context
        #         t = threading.Thread(target=self.handler, args=(self.handler, newMsg, context))
        #         t.setDaemon(True)
        #         t.start()
                # py_runner.run_custom_func(handler, result)
            # compare funcid, longlived是否监控，进程id怎么感知？cgroup的行为
        else:
            pass

    def send_message(self, data):
        self.send(data)

    def request(self, params, cb):
        logging.debug("request:%s", params)
        if "body" in params:
            payload = params["body"]
            if payload is None:
                params["body"] = ''

            if not isinstance(params["body"], str):
                payload = str(payload)
            payload = payload.encode("utf-8")
            params["body"] = base64.standard_b64encode(payload).decode("utf-8")
        uuid = utils.get_uuid()
        self.pendingCalls[uuid] = cb
        request = {"tag": uuid, "cmd": "request", "options": params}
        self.send_message(json.dumps(request))

    #id = pk_dn
    def setProperty(self, id, values, cb):
        uuid = utils.get_uuid()
        self.pendingCalls[uuid] = cb
        request = {"tag": uuid, "cmd": "setProperty", "id": id, "values": values}
        # http format
        # payload = {"header": "method": "POST", "body": payload}
        # request = {"tag": uuid, "cmd": "request", "options": payload}
        self.send_message(json.dumps(request))

    def getProperty(self, id, keys, cb):
        uuid = utils.get_uuid()
        self.pendingCalls[uuid] = cb
        request = {"tag": uuid, "cmd": "getProperty", "id": id, "keys": keys}
        self.send_message(json.dumps(request))

    def callService(self, id, service, params, cb):
        uuid = utils.get_uuid()
        self.pendingCalls[uuid] = cb
        request = {"tag": uuid, "cmd": "callService", "id": id, "service": service, "params": params}
        self.send_message(json.dumps(request))

    def sendJobExecuteStatus(self, jobId, status):
        request = {"cmd": "sendJobExecuteStatus", "id": jobId, "status": status}
        self.send_message(json.dumps(request))

    def getDeviceList(self, status, cb):
        uuid = utils.get_uuid()
        self.pendingCalls[uuid] = cb
        request = {"tag": uuid, "cmd": "getDeviceList", "status": status}
        self.send_message(json.dumps(request))

    def getDeviceProfile(self, id, cb):
        uuid = utils.get_uuid()
        self.pendingCalls[uuid] = cb
        request = {"tag": uuid, "cmd": "getDeviceProfile", "id": id}
        self.send_message(json.dumps(request))

    def publish(self, wellknownName, _interface, objectPath, messageName, payload, cb):
        uuid = utils.get_uuid()
        self.pendingCalls[uuid] = cb

        request = {"tag": uuid, "cmd": "publish", "wellknownName": wellknownName, "_interface": _interface,
                    "objectPath": objectPath, "messageName": messageName, "payload": payload}
        self.send_message(json.dumps(request))

    def getConfig(self, key, cb):
        uuid = utils.get_uuid()
        self.pendingCalls[uuid] = cb
        request = {"tag": uuid, "cmd": "getConfig", "key": key}
        self.send_message(json.dumps(request))

def get_ws_client():
    return WS_Client('ws://127.0.0.1:8089/')

try:
    ws = get_ws_client()
except Exception as user_ex:
    logging.error(user_ex)

if __name__ == '__main__':
    try:
        ws = WS_Client('ws://127.0.0.1:8089/', None)
        # ws.connect()
        ws.send_message("hello")
        ws.getConfig("config", None)
        ws.getDeviceProfile(1, None)
        ws.getDeviceList(1, None)
        ws.run_forever()
    except KeyboardInterrupt:
        ws.close()
