'use strict';
const child_process = require('child_process');
const path = require('path');
const Log = require("../utils/log");
const TAG = "[Scheduler]";
const {PROJECT_PATH} = require('../const');
const dirname = path.join(PROJECT_PATH, 'scheduler');
const constant = require("../utils/constant");
const broker = require("../components/broker");
var longliveProcess = {};

function Scheduler() {

};

Scheduler.prototype.schedule = function (jobType, jobInfo, jobConfig) {
  Log.i(TAG, "schedule begin...");
  var entry = path.join(PROJECT_PATH, 'runner/index.js');
  var jobInfoStr = JSON.stringify(jobInfo);
  var envPath = path.join(dirname, 'before_start.sh');
  var cmd = " " + jobType + " " + jobInfoStr;
  Log.i(TAG, 'SCHEDULER  start to execute:', cmd);

  process.env.NODE_PATH = PROJECT_PATH + "/node_modules:" + PROJECT_PATH + "/runner/developer:" + PROJECT_PATH + "/../../lib";
  process.env.LD_LIBRARY_PATH = PROJECT_PATH + "/build-env/encrypt:" + PROJECT_PATH + "/../../lib";
  const defaults = {
    cwd: undefined,
    env: process.env
  };
  var runner;
  if (jobType === 'script') {
    var jobConfigStr = JSON.stringify(jobConfig);
    var pyentry = path.join(PROJECT_PATH, 'pyrunner/py_runtime.py');
    Log.i(TAG, 'PROJECT_PATH:' + PROJECT_PATH);
    Log.i(TAG, 'jobInfoStr:' + jobInfoStr);
    Log.i(TAG, 'jobConfigStr:' + jobConfigStr);
    runner = child_process.spawn(PROJECT_PATH + "cgctl", [jobConfigStr, PROJECT_PATH + "build-env/node/bin/node", entry, '--type', jobType, '--info', jobInfoStr, '--config', jobConfigStr], defaults);
    // runner = child_process.spawn(PROJECT_PATH + "cgctl", [jobConfigStr, PROJECT_PATH + "py3_venv/bin/python3", pyentry, '--type', jobType, '--info', jobInfoStr, '--config', jobConfigStr], defaults);
  } else {
    Log.i(TAG, "schedule IFTTT begin...");
    //runner = child_process.spawn(PROJECT_PATH + "/build-env/node/bin/node", [entry, '--type', jobType, '--info', jobInfoStr], defaults);
    runner = child_process.spawn("node", [entry, '--type', jobType, '--info', jobInfoStr], defaults);
  }
  if (jobType === 'script' && jobInfo[constant.CONFIG_RUNMODE] === constant.CONFIG_RUNMODE_LONGLIVE) {
    longliveProcess[jobInfo.jobId] = runner;
  }
  runner.stdout.on('data', function (data) {
    Log.i(TAG, 'Runner.stdout:', data);
  });

  runner.stderr.on('data', function (data) {
    Log.e(TAG, 'Runner.stderr:', data);
  });

  runner.on('close', function (code, signal) {
    Log.w(TAG, 'Runner closed, code:' + code + " signal =" + signal + " jobId = " + jobInfo.jobId + " longliveProcess size = " + Object.keys(longliveProcess).length);
    Log.w(TAG, "close jobid " + jobInfo.jobId, "sceneId", jobInfo.sceneId);
    if (longliveProcess[jobInfo.jobId] != undefined) {
      delete(longliveProcess[jobInfo.jobId]);
      Log.i("emit close event");
      broker.publish("close", jobInfo);
    }
  });
};

Scheduler.prototype.stopListenLLProcess = function (jobId) {
  delete(longliveProcess[jobId]);
};

Scheduler.prototype.getLLProcess = function (jobId) {
  return longliveProcess[jobId];
};

module.exports = new Scheduler();
